package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.sys_organization.*;
import pps.core.common.constant.BusinessConstant;
import pps.core.system.entity.*;
import pps.core.system.mapper.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@XText("组织机构接口")
@XService
public class SysOrganizationCloudServiceImpl implements SysOrganizationCloudService {

    @Override
    public XSingleResult<GetSysOrganizationViewOutput> getSysOrganizationById(XContext context, GetSysOrganizationViewInput input) {
        SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
        QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrganizationEnt::getId, input.getId());
        SysOrganizationEnt ent = mapper.selectOne(queryWrapper);
        GetSysOrganizationViewOutput output = new GetSysOrganizationViewOutput();
        XCopyUtils.copyObject(ent, output);
        return XSingleResult.success(output);
    }

    @Override
    public XSingleResult<GetSysOrganizationViewOutput> getOuByUserId(XContext context, GetSysOrganizationViewInput input) {
        SysUserOrganizationRelMapper mapper = context.getBean(SysUserOrganizationRelMapper.class);
        SysOrganizationMapper sysOrganizationMapper = context.getBean(SysOrganizationMapper.class);
        SysOrganizationRelMapper sysOrganizationRelMapper = context.getBean(SysOrganizationRelMapper.class);
        QueryWrapper<SysUserOrganizationRelEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(SysUserOrganizationRelEnt::getUserId, input.getUserId())
                .le(SysUserOrganizationRelEnt::getEffectTime, new Date())
                .ge(SysUserOrganizationRelEnt::getEndTime, new Date());
        List<SysUserOrganizationRelEnt> relEnts = mapper.selectList(wrapper);
        GetSysOrganizationViewOutput output = new GetSysOrganizationViewOutput();
        if (!CollectionUtils.isEmpty(relEnts)) {
            if (XStringUtils.isNotEmpty(relEnts.get(0).getOuId())) {
                QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(SysOrganizationEnt::getId, relEnts.get(0).getOuId())
                        .eq(SysOrganizationEnt::getIsDeleted, 1);
                SysOrganizationEnt sysOrganizationEnt = sysOrganizationMapper.selectOne(queryWrapper);
                XCopyUtils.copyObject(sysOrganizationEnt, output);
                output.setOuId(relEnts.get(0).getOuId());
                QueryWrapper<SysOrganizationRelEnt> qw = new QueryWrapper<>();
                qw.lambda().eq(SysOrganizationRelEnt::getOuId, relEnts.get(0).getOuId())
                        .le(SysOrganizationRelEnt::getEffectTime, new Date())
                        .ge(SysOrganizationRelEnt::getEndTime, new Date());
                List<SysOrganizationRelEnt> sysOrganizationRelEnts = sysOrganizationRelMapper.selectList(qw);
                if (!CollectionUtils.isEmpty(sysOrganizationRelEnts)) {
                    output.setOuPath(sysOrganizationRelEnts.get(0).getOuIdPath());
                }
            }
        }
        return XSingleResult.success(output);
    }

    @XText("根据用户id获取用户组织机构")
    public XListResult<OuIdAndOutNameOutput> getOrgListByUserId(XContext context, QueryUserIdInput input) {
        SysUserOrganizationRelViewMapper mapper = context.getBean(SysUserOrganizationRelViewMapper.class);
        List<SysUserOrganizationRelView> list = mapper.getOrgListByUserId(input.getUserId());
        List<OuIdAndOutNameOutput> outputs = XCopyUtils.copyNewList(list, OuIdAndOutNameOutput.class);
        return XListResult.success(outputs);
    }

    @XText("根据组织机构id获取所有的子级组织机构")
    public XListResult<GetOuListTreeOutput> getAllOuListByOuId(XContext context, GetAllOuListByOuIdInput input) {
        List<GetOuListTreeOutput> outputs = new ArrayList<>();
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        view.setIsDeleted(1);
        view.setId(input.getOuId());
        List<SysOrganizationView> list = mapper.selectAllListByOuId(view);
        if (list.size() > 0) {
            for (SysOrganizationView item : list) {
                GetOuListTreeOutput out = new GetOuListTreeOutput();
                out.setOuId(item.getId());
                out.setOuName(item.getOuName());
                out.setShortName(item.getShortName());
                out.setParentOuId(item.getParentOuId());
                outputs.add(out);
            }
        }
        return XListResult.success(outputs);
    }

    @XText("根据组织机构id获取所有的子级组织机构")
    public XListResult<GetOuListTreeOutput> getOuListByOuIds(XContext context, GetOuListByOuIdsInput input) {
        if (CollectionUtils.isEmpty(input.getOuIdList())) return XListResult.success(new ArrayList<>());
        List<GetOuListTreeOutput> outputs = new ArrayList<>();
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        List<SysOrganizationView> list = mapper.selectListByOuIds(input.getOuIdList());
        if (list.size() > 0) {
            for (SysOrganizationView item : list) {
                GetOuListTreeOutput out = new GetOuListTreeOutput();
                out.setOuId(item.getId());
                out.setOuName(item.getOuName());
                out.setShortName(item.getShortName());
                out.setParentOuId(item.getParentOuId());
                outputs.add(out);
            }
        }
        return XListResult.success(outputs);
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> getAllListByOuId(XContext context, GetAllOuListByOuIdInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        view.setIsDeleted(1);
        view.setId(input.getOuId());
        view.setOuLevel(input.getOuLevel());
        List<SysOrganizationView> list = mapper.selectAllListByOuId(view);
        List<GetSysOrganizationViewOutput> viewList = XCopyUtils.copyNewList(list, GetSysOrganizationViewOutput.class);
        return XListResult.success(viewList);
    }

    @Override
    public XListResult<GetSysOrganizationViewOutput> getPlantList(XContext context, GetSysOrganizationViewInput input) {
        SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
        List<SysOrganizationEnt> plant = mapper.selectList(new LambdaQueryWrapper<SysOrganizationEnt>()
                .eq(SysOrganizationEnt::getIsDeleted, BusinessConstant.ONE)
                .eq(SysOrganizationEnt::getOuLevel, BusinessConstant.PLANT_FLAG)
                .orderByAsc(SysOrganizationEnt::getCreateTime)
        );
        return XListResult.success(XCopyUtils.copyNewList(plant, GetSysOrganizationViewOutput.class));
    }
}