package pps.core.base.service;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.util.CollectionUtils;
import pps.core.base.entity.BaseDataEnt;
import pps.core.base.entity.BaseDataImportLogEnt;
import pps.core.base.entity.BaseDataView;
import pps.core.base.entity.BaseModelValEnt;
import pps.core.base.enums.ImportFieldDic;
import pps.core.base.mapper.BaseDataImportLogMapper;
import pps.core.base.mapper.BaseDataMapper;
import pps.core.base.mapper.BaseDataViewMapper;
import pps.core.base.mapper.BaseModelValMapper;
import pps.core.base.service.data.base_data.*;
import pps.core.base.service.data.base_line.ImportFileInput;
import pps.core.base.utils.HttpUtils;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.ExcelUtils;
import pps.core.common.utils.ExportExcelUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.tool.XStorageTool;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XJsonUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.*;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@XService
public class BaseDataService {
    @XApiPost
    public XServiceResult createBaseData(XContext context, CreateBaseDataInput input) {
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        BaseDataEnt entity = new BaseDataEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult updateBaseData(XContext context, UpdateBaseDataInput input) {
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getId, input.getId());
        BaseDataEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult deleteBaseData(XContext context, DeleteBaseDataInput input) {
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getId, input.getId());
        BaseDataEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiGet
    public XSingleResult<GetBaseDataOutput> getBaseData(XContext context, GetBaseDataInput input) {
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getId, input.getId());
        BaseDataEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseDataOutput output = new GetBaseDataOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiGet
    public XListResult<QueryBaseDataOutput> queryBaseDataCopy(XContext context, GetBaseDataInput input) {
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        Date beginTime;
        Date endTime;
        if (XStringUtils.isEmpty(input.getDataDate())) {
            beginTime = XDateUtils.getDayBeginTime(new Date());
            endTime = XDateUtils.getDayEndTime(new Date());
            queryWrapper.lambda().ge(BaseDataEnt::getDataDate, beginTime);
            queryWrapper.lambda().le(BaseDataEnt::getDataDate, endTime);
        } else {
            if (input.getDataDate().contains("@")) {
                String[] dataDates = input.getDataDate().split("@");
                beginTime = XDateUtils.getDayBeginTime(XDateUtils.parse(dataDates[0]));
                endTime = XDateUtils.getDayEndTime(XDateUtils.parse(dataDates[1]));
                queryWrapper.lambda().ge(BaseDataEnt::getDataDate, beginTime);
                queryWrapper.lambda().le(BaseDataEnt::getDataDate, endTime);
            } else {
                beginTime = XDateUtils.getDayBeginTime(XDateUtils.parse(input.getDataDate()));
                endTime = XDateUtils.getDayEndTime(XDateUtils.parse(input.getDataDate()));
                queryWrapper.lambda().ge(BaseDataEnt::getDataDate, beginTime);
                queryWrapper.lambda().le(BaseDataEnt::getDataDate, endTime);
            }
        }
        queryWrapper.lambda().eq(BaseDataEnt::getLineId, input.getLineId());
        queryWrapper.lambda().orderByAsc(BaseDataEnt::getDataDate);
        List<BaseDataEnt> list = mapper.selectList(queryWrapper);
        CalcBaseDataInput calcBaseDataInput = new CalcBaseDataInput("", input.getKeyName(), input.getLineId(), "train_pre", XDateUtils.getString(beginTime), XDateUtils.getString(endTime));
        Map<String, Object> map = getCalcData(context, calcBaseDataInput);
        list.stream().forEach(item -> {
            String dataDate = XDateUtils.getString(item.getDataDate());
            if (map.get(dataDate) != null &&
                    XStringUtils.isNotEmpty(map.get(dataDate).toString())) {
                item.setPredictedPower(new BigDecimal(map.get(dataDate).toString()));
            }
        });
        List<QueryBaseDataOutput> outputs = XCopyUtils.copyNewList(list, QueryBaseDataOutput.class);
        return XListResult.success(outputs);
    }

    @XApiGet
    public XListResult<QueryBaseDataOutput> queryBaseData(XContext context, GetBaseDataInput input) {
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        BaseDataImportLogMapper logMapper = context.getBean(BaseDataImportLogMapper.class);
        QueryWrapper<BaseDataImportLogEnt> logEntQueryWrapper = new QueryWrapper<>();
        logEntQueryWrapper.lambda().eq(BaseDataImportLogEnt::getLineId, input.getLineId())
                .orderByDesc(BaseDataImportLogEnt::getCreateTime)
                .last(BusinessConstant.LAST_LIMIT);
        BaseDataImportLogEnt ent = logMapper.selectOne(logEntQueryWrapper);
        List<QueryBaseDataOutput> outputs = new ArrayList<>();
        if (null != ent) {
            queryWrapper.lambda().ge(BaseDataEnt::getDataDate, ent.getBeginDate());
            queryWrapper.lambda().le(BaseDataEnt::getDataDate, ent.getEndDate());
            queryWrapper.lambda().eq(BaseDataEnt::getLineId, input.getLineId());
            queryWrapper.lambda().orderByAsc(BaseDataEnt::getDataDate);
            List<BaseDataEnt> list = mapper.selectList(queryWrapper);
            outputs = XCopyUtils.copyNewList(list, QueryBaseDataOutput.class);
        }
        return XListResult.success(outputs);
    }

    @XApiGet
    @XText("获取预测数据")
    public XListResult<GetBaseDataOutput> queryPreBaseData(XContext context, GetBaseDataInput input) {
        BaseDataImportLogMapper logMapper = context.getBean(BaseDataImportLogMapper.class);
        QueryWrapper<BaseDataImportLogEnt> logEntQueryWrapper = new QueryWrapper<>();
        logEntQueryWrapper.lambda().eq(BaseDataImportLogEnt::getLineId, input.getLineId())
                .orderByDesc(BaseDataImportLogEnt::getCreateTime)
                .last(BusinessConstant.LAST_LIMIT);
        BaseDataImportLogEnt ent = logMapper.selectOne(logEntQueryWrapper);
        List<GetBaseDataOutput> outputs = new ArrayList<>();
        if (ent != null) {
            BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
            QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().ge(BaseDataEnt::getDataDate, ent.getBeginDate());
            queryWrapper.lambda().le(BaseDataEnt::getDataDate, ent.getEndDate());
            queryWrapper.lambda().eq(BaseDataEnt::getLineId, input.getLineId());
            queryWrapper.lambda().orderByAsc(BaseDataEnt::getDataDate);
            List<BaseDataEnt> list = mapper.selectList(queryWrapper);
            CalcBaseDataInput calcBaseDataInput = new CalcBaseDataInput("", input.getKeyName(), input.getLineId(), "train_pre", XDateUtils.getString(ent.getBeginDate()), XDateUtils.getString(ent.getEndDate()));
            Map<String, Object> map = getCalcData(context, calcBaseDataInput);
//            for(String key : map.keySet()) {
//                GetBaseDataOutput detailOutput = new GetBaseDataOutput();
//                detailOutput.setDataDate(XDateUtils.parse(key));
//                detailOutput.setPredictedPower(new BigDecimal(map.get(key).toString()));
//                outputs.add(detailOutput);
//            }
            list.forEach(item -> {
                String dataDate = XDateUtils.getString(item.getDataDate());
                if (map.get(dataDate) != null &&
                        XStringUtils.isNotEmpty(map.get(dataDate).toString())) {
                    item.setPredictedPower(new BigDecimal(map.get(dataDate).toString()));
                }
            });
            outputs = XCopyUtils.copyNewList(list, GetBaseDataOutput.class);
        }
        return XListResult.success(outputs);
    }

    @XApiGet
    public XPageResult<QueryBaseDataOutput> queryBaseDataList(XContext context, QueryBaseDataInput input) {
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getLineId, input.getLineId())
                .eq(BaseDataEnt::getDataDate, input.getDataDate());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseDataOutput.class);
    }

    @XApiGet
    public XPageResult<QueryBaseDataOutput> dynamicQueryBaseData(XContext context, CustomQueryInput input) {
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        return XMapperHelper.query(mapper, input, BaseDataEnt.class, QueryBaseDataOutput.class);
    }

    @XText("下载导入模板")
    @XApiGet
    public XFileResult downloadImportTemplate(XContext context) {
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/导入模板_" + XDateUtils.getString(new Date(), "yyyy-MM-dd") + ".xlsx");
        String templateFilePath = "template/import_template.xlsx";
        InputStream templateInputStream = this.getClass().getClassLoader().getResourceAsStream(templateFilePath);
        byte[] excelContent = ExportExcelUtils.genSingleExcelFileData(templateInputStream, null);
        try {
            ExportExcelUtils.outputFileData(excelContent, fileSavePath);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    @XText("新增导入")
    @XApiUpload
    public XServiceResult importFile(XContext context, ImportFileInput input) {
        List<String> headerList = getExcelHeaderList();
        List<BaseDataView> list = new ArrayList<>();
        List<Map<String, Object>> mapList = ExcelUtils.readExcel(input.getFile().getInputStream(), input.getFile().getFileName(), headerList, 0);
        try {
            List<BaseDataView> dataList = new ArrayList<>();
            Date minDate = null;
            Date maxDate = null;
            for (Map<String, Object> objectMap : mapList) {
                BaseDataView dataView = BeanUtil.toBean(objectMap, BaseDataView.class);
                dataView.setOuId(input.getWellOuId());
                dataView.setLineId(input.getLineId());
                if (dataView.getDataDate() != null) {
                    if (minDate == null) {
                        minDate = dataView.getDataDate();
                        maxDate = dataView.getDataDate();
                    }
                    if (minDate.compareTo(dataView.getDataDate()) > 0) {
                        minDate = dataView.getDataDate();
                    } else if (maxDate.compareTo(dataView.getDataDate()) < 0) {
                        maxDate = dataView.getDataDate();
                    }
                }
                dataList.add(dataView);
            }
            if (minDate == null) {
                return XServiceResult.error(1000, "日期错误");
            }
            BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
            QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseDataEnt::getLineId, input.getLineId())
                    .ge(BaseDataEnt::getDataDate, minDate)
                    .le(BaseDataEnt::getDataDate, maxDate);
            mapper.delete(queryWrapper);
            BaseDataViewMapper viewMapper = context.getBean(BaseDataViewMapper.class);
            viewMapper.batchInsert(dataList);
            BaseDataImportLogMapper logMapper = context.getBean(BaseDataImportLogMapper.class);
            BaseDataImportLogEnt ent = new BaseDataImportLogEnt();
            ent.setOuId(input.getWellOuId());
            ent.setLineId(input.getLineId());
            ent.setBeginDate(minDate);
            ent.setEndDate(maxDate);
            ent.setCreateTime(new Date());
            logMapper.insert(ent);
            //进行训练
//            CalcBaseDataInput calcBaseDataInput = new CalcBaseDataInput("" , input.getLineId() , "train_val" , XDateUtils.getString(minDate) , XDateUtils.getString(maxDate));
//            doTrainData(context , calcBaseDataInput);
        } catch (Exception e) {
            return XServiceResult.error(-1, e.getMessage());
        }
        return XServiceResult.OK;
    }

    private List<String> getExcelHeaderList() {
        List<Map<String, Object>> fieldList = new ArrayList<>();
        for (ImportFieldDic value : ImportFieldDic.values()) {
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("col", value.getCol());
            map.put("sort", value.getSort());
            fieldList.add(map);
        }
        fieldList.sort(new Comparator<Map<String, Object>>() {
            @Override
            public int compare(Map<String, Object> o1, Map<String, Object> o2) {
                return ((Integer) o1.get("sort")).compareTo((Integer) o2.get("sort"));
            }
        });
        return fieldList.stream().map(x -> ((String) x.get("col"))).collect(Collectors.toList());
    }

    @XText("获取预算")
    @XApiGet
    public XServiceResult calcBaseData(XContext context, CalcBaseDataInput calcBaseDataInput) {
        getCalcData(context, calcBaseDataInput);
        return XServiceResult.OK;
    }

    public Map<String, Object> getCalcData(XContext context, CalcBaseDataInput calcBaseDataInput) {
        Map<String, Object> returnMap = new HashMap<>();
        try {
            String jsonString = XJsonUtils.toJson(calcBaseDataInput);
            context.getLogger().info(jsonString);
            Object ret = HttpUtils.send2("http://121.36.43.217:8098/model/predict",
                    jsonString);
            returnMap = XJsonUtils.toMap(ret.toString());
            return returnMap;
        } catch (Exception e) {
            return returnMap;
        }
    }

    public void doTrainData(XContext context, CalcBaseDataInput calcBaseDataInput) {
        try {
            String jsonString = XJsonUtils.toJson(calcBaseDataInput);
            context.getLogger().info(jsonString);
            Object ret = HttpUtils.send2("http://121.36.43.217:8098/model/train",
                    jsonString);
        } catch (Exception e) {
            throw new RuntimeException("调用预测接口失败");
        }
    }

    @XApiPost
    @XText("预测算法训练")
    public XServiceResult trainBaseData(XContext context, TrainBaseModelValInput input) {
        BaseDataImportLogMapper logMapper = context.getBean(BaseDataImportLogMapper.class);
        QueryWrapper<BaseDataImportLogEnt> logEntQueryWrapper = new QueryWrapper<>();
        logEntQueryWrapper.lambda().eq(BaseDataImportLogEnt::getLineId, input.getLineId())
                .orderByDesc(BaseDataImportLogEnt::getCreateTime)
                .last(BusinessConstant.LAST_LIMIT);
        BaseDataImportLogEnt ent = logMapper.selectOne(logEntQueryWrapper);
        if (ent == null)
            return XServiceResult.OK;
        if (!CollectionUtils.isEmpty(input.getKeyNames())) {
            for (String keyName : input.getKeyNames()) {
                CalcBaseDataInput calcBaseDataInput = new CalcBaseDataInput("", keyName, input.getLineId(), "train_val", XDateUtils.getString(ent.getBeginDate()), XDateUtils.getString(ent.getEndDate()));
                doTrainData(context, calcBaseDataInput);
            }
        } else {
            CalcBaseDataInput calcBaseDataInput = new CalcBaseDataInput("", input.getKeyName(), input.getLineId(), "train_val", XDateUtils.getString(ent.getBeginDate()), XDateUtils.getString(ent.getEndDate()));
            doTrainData(context, calcBaseDataInput);
        }
        return XServiceResult.OK;
    }

    @XApiGet
    @XText("获取最新算法训练数据")
    public XSingleResult<TrainBaseModelValOutput> getTrainBaseData(XContext context, GetTrainBaseModelValInput input) {
        BaseModelValMapper baseModelValMapper = context.getBean(BaseModelValMapper.class);
        if (input.getLineId() == null)
            return XSingleResult.error(500, "请选择线路");
        if (XStringUtils.isEmpty(input.getKeyNames()))
            return XSingleResult.error(500, "请选择模型");
        QueryWrapper<BaseModelValEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseModelValEnt::getLineId, input.getLineId())
                .eq(BaseModelValEnt::getKeyname, input.getKeyNames())
                .orderByDesc(BaseModelValEnt::getCreatetime).last(BusinessConstant.LAST_LIMIT);
        BaseModelValEnt baseModelValEnt = baseModelValMapper.selectOne(queryWrapper);
        TrainBaseModelValOutput output = XCopyUtils.copyNewObject(baseModelValEnt, TrainBaseModelValOutput.class);
        if (baseModelValEnt != null && XStringUtils.isNotEmpty(output.getYtrue()) && XStringUtils.isNotEmpty(output.getYpred())) {
            Map<String, Object> ytrueMap = XJsonUtils.toMap(output.getYtrue().replace("\\", ""));
            Map<String, Object> yPreMap = XJsonUtils.toMap(output.getYpred().replace("\\", ""));
            List<TrainBaseModelValDetailOutput> list = new ArrayList<>();
            for (String key : ytrueMap.keySet()) {
                TrainBaseModelValDetailOutput detailOutput = new TrainBaseModelValDetailOutput();
                Date date = new Date(Long.valueOf(key));
                detailOutput.setKey(XDateUtils.getString(date));
                detailOutput.setPower(ytrueMap.get(key).toString());
                if (yPreMap.get(key) != null)
                    detailOutput.setPredictedPower(yPreMap.get(key).toString());
                list.add(detailOutput);
            }
            output.setList(list);
        } else {
            output = new TrainBaseModelValOutput();
        }
        return XSingleResult.success(output);
    }
}