package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import pps.cloud.space.service.ISpaceInstitutionDetailCloudService;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.sys_organization.GetOuListByOuIdsInput;
import pps.cloud.system.service.data.sys_organization.GetOuListTreeOutput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.core.base.entity.*;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_power_line.*;
import pps.core.base.service.data.base_power_line_diesel.GetBasePowerLineDieselViewOutput;
import pps.core.base.service.data.base_power_line_plant.GetBasePowerLinePlantViewOutput;
import pps.core.base.service.data.base_power_line_storage.GetBasePowerLineStorageViewOutput;
import pps.core.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.core.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.core.base.service.data.base_power_line_wellhead.GetBasePowerLineWellheadViewOutput;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 输电线路配置模块
 *
 * @author ZWT
 * @date 2023/08/25
 */
@XService
@XText("输电线路配置模块")
public class BasePowerLineService {

    /**
     * 输电线路配置--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiPost
    @XText("输电线路配置--新增")
    public XServiceResult createBasePowerLine(XContext context, CreateBasePowerLineInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        return XTransactionHelper.begin(context, () -> {
            BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
            BasePowerLineEnt entity = XCopyUtils.copyNewObject(input, BasePowerLineEnt.class);
            //新增输电线路
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            String lineId = entity.getId();
            String ouId = entity.getOuId();
            //新增关联信息
            this.saveLineRelation(context, lineId, ouId, session, input.getWellheadInputs(), input.getPlantInputs(), input.getStorageInputs(), input.getDieselInputs());
            return XServiceResult.OK;
        });
    }

    /**
     * 输电线路配置--修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiPost
    @XText("输电线路配置--修改")
    public XServiceResult updateBasePowerLine(XContext context, UpdateBasePowerLineInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        String lineId = input.getId();
        String ouId = input.getOuId();
        return XTransactionHelper.begin(context, () -> {
            BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
            BasePowerLineEnt entity = this.selectOneByLineId(lineId, mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //删除井口关联信息
            BasePowerLineWellheadMapper wellheadMapper = context.getBean(BasePowerLineWellheadMapper.class);
            wellheadMapper.delete(new LambdaQueryWrapper<BasePowerLineWellheadEnt>()
                    .eq(BasePowerLineWellheadEnt::getLineId, lineId)
            );
            //删除光伏关联信息
            BasePowerLinePlantMapper plantMapper = context.getBean(BasePowerLinePlantMapper.class);
            plantMapper.delete(new LambdaQueryWrapper<BasePowerLinePlantEnt>()
                    .eq(BasePowerLinePlantEnt::getLineId, lineId)
            );
            //删除储能关联信息
            BasePowerLineStorageMapper storageMapper = context.getBean(BasePowerLineStorageMapper.class);
            storageMapper.delete(new LambdaQueryWrapper<BasePowerLineStorageEnt>()
                    .eq(BasePowerLineStorageEnt::getLineId, lineId)
            );
            //删除柴发关联信息
            BasePowerLineDieselMapper dieselMapper = context.getBean(BasePowerLineDieselMapper.class);
            dieselMapper.delete(new LambdaQueryWrapper<BasePowerLineDieselEnt>()
                    .eq(BasePowerLineDieselEnt::getLineId, lineId)
            );
            //新增关联信息
            this.saveLineRelation(context, lineId, ouId, session, input.getWellheadInputs(), input.getPlantInputs(), input.getStorageInputs(), input.getDieselInputs());
            XCopyUtils.copyObject(input, entity);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 输电线路配置--删除
     * (如果被引用,则不能删除)
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiPost
    @XText("输电线路配置--删除")
    public XServiceResult deleteBasePowerLine(XContext context, DeleteBasePowerLineInput input) {
        String lineId = input.getId();
        if (this.checkLineIsReference(context, lineId)) {
            return XServiceResult.error(context, BusinessError.LineReference);
        }
        return XTransactionHelper.begin(context, () -> {
            BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
            BasePowerLineEnt entity = this.selectOneByLineId(lineId, mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //删除线路配置
            mapper.update(null, new LambdaUpdateWrapper<BasePowerLineEnt>()
                    .eq(BaseModel::getId, lineId)
                    .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
            );
            //删除井口关联信息
            BasePowerLineWellheadMapper wellheadMapper = context.getBean(BasePowerLineWellheadMapper.class);
            wellheadMapper.update(null, new LambdaUpdateWrapper<BasePowerLineWellheadEnt>()
                    .eq(BasePowerLineWellheadEnt::getLineId, lineId)
                    .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
            );
            //删除光伏关联信息
            BasePowerLinePlantMapper plantMapper = context.getBean(BasePowerLinePlantMapper.class);
            plantMapper.update(null, new LambdaUpdateWrapper<BasePowerLinePlantEnt>()
                    .eq(BasePowerLinePlantEnt::getLineId, lineId)
                    .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
            );
            //删除储能关联信息
            BasePowerLineStorageMapper storageMapper = context.getBean(BasePowerLineStorageMapper.class);
            storageMapper.update(null, new LambdaUpdateWrapper<BasePowerLineStorageEnt>()
                    .eq(BasePowerLineStorageEnt::getLineId, lineId)
                    .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
            );
            //删除柴发关联信息
            BasePowerLineDieselMapper dieselMapper = context.getBean(BasePowerLineDieselMapper.class);
            dieselMapper.update(null, new LambdaUpdateWrapper<BasePowerLineDieselEnt>()
                    .eq(BasePowerLineDieselEnt::getLineId, lineId)
                    .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
            );
            return XServiceResult.OK;
        });
    }

    /**
     * 输电线路配置--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePowerLineViewOutput}>
     */
    @XApiGet
    @XText("输电线路配置--详情")
    public XSingleResult<GetBasePowerLineViewOutput> getBasePowerLineView(XContext context, GetBasePowerLineInput input) {
        String lineId = input.getId();
        BasePowerLineViewMapper mapper = context.getBean(BasePowerLineViewMapper.class);
        BasePowerLineView basePowerLineView = new BasePowerLineView();
        basePowerLineView.setId(lineId);
        BasePowerLineView entity = mapper.selectOne(basePowerLineView);
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePowerLineViewOutput output = XCopyUtils.copyNewObject(entity, GetBasePowerLineViewOutput.class);
        //设置关联井口
        BasePowerLineWellheadViewMapper wellheadViewMapper = context.getBean(BasePowerLineWellheadViewMapper.class);
        List<BasePowerLineWellheadView> wellheadViewList = wellheadViewMapper.selectList(BasePowerLineWellheadView.builder()
                .lineId(lineId)
                .build());
        if (CollUtil.isNotEmpty(wellheadViewList)) {
            output.setWellheadInputs(XCopyUtils.copyNewList(wellheadViewList, GetBasePowerLineWellheadViewOutput.class));
        } else {
            output.setWellheadInputs(Collections.emptyList());
        }
        //设置关联光伏
        BasePowerLinePlantViewMapper plantViewMapper = context.getBean(BasePowerLinePlantViewMapper.class);
        List<BasePowerLinePlantView> plantViewList = plantViewMapper.selectList(BasePowerLinePlantView.builder()
                .lineId(lineId)
                .build());
        if (CollUtil.isNotEmpty(plantViewList)) {
            output.setPlantInputs(XCopyUtils.copyNewList(plantViewList, GetBasePowerLinePlantViewOutput.class));
        } else {
            output.setPlantInputs(Collections.emptyList());
        }
        //设置关联储能
        BasePowerLineStorageViewMapper storageViewMapper = context.getBean(BasePowerLineStorageViewMapper.class);
        List<BasePowerLineStorageView> storageViewList = storageViewMapper.selectList(BasePowerLineStorageView.builder()
                .lineId(lineId)
                .build());
        if (CollUtil.isNotEmpty(storageViewList)) {
            output.setStorageInputs(XCopyUtils.copyNewList(storageViewList, GetBasePowerLineStorageViewOutput.class));
        } else {
            output.setStorageInputs(Collections.emptyList());
        }
        //设置关联柴发
        BasePowerLineDieselViewMapper dieselViewMapper = context.getBean(BasePowerLineDieselViewMapper.class);
        List<BasePowerLineDieselView> dieselViewList = dieselViewMapper.selectList(BasePowerLineDieselView.builder()
                .lineId(lineId)
                .build());
        if (CollUtil.isNotEmpty(dieselViewList)) {
            output.setDieselInputs(XCopyUtils.copyNewList(dieselViewList, GetBasePowerLineDieselViewOutput.class));
        } else {
            output.setDieselInputs(Collections.emptyList());
        }
        Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
        Map<String, String> lineTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.LINE_TYPE);
        output.setGridTypeName(
                gridTypeMap.get(output.getGridTypeKey())
        );
        output.setLineTypeName(
                lineTypeMap.get(output.getLineTypeKey())
        );
        output.setOuName(this.getSysOrgNameById(context, output.getOuId()));
        return XSingleResult.success(output);
    }

    /**
     * 输电线路配置--分页列表接口
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBasePowerLineViewOutput}>
     */
    @XApiGet
    @XText("输电线路配置--分页列表")
    public XPageResult<QueryBasePowerLineViewOutput> queryBasePowerLineView(XContext context, QueryBasePowerLineViewInput input) {
        BasePowerLineViewMapper mapper = context.getBean(BasePowerLineViewMapper.class);
        BasePowerLineView lineView = XCopyUtils.copyNewObject(input, BasePowerLineView.class);
        lineView.setOuList(ServiceUtil.getOrgIdsByPath(context, lineView.getOuId()));
        PageMethod.startPage(input.getPage(), input.getLimit());
        List<BasePowerLineView> list = mapper.selectList(lineView);
        PageInfo<BasePowerLineView> pageInfo = new PageInfo<>(list);
        List<QueryBasePowerLineViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePowerLineViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> lineTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.LINE_TYPE);
            for (QueryBasePowerLineViewOutput output : outputs) {
                output.setGridTypeName(
                        gridTypeMap.get(output.getGridTypeKey())
                );
                output.setLineTypeName(
                        lineTypeMap.get(output.getLineTypeKey())
                );
            }
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 输电线路配置--列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineOutput}>
     */
    @XApiGet
    @XText("输电线路配置--列表")
    public XListResult<DynamicQueryBasePowerLineOutput> queryBasePowerLineList(XContext context, DynamicQueryBasePowerLineInput input) {
        String lineId = input.getId();
        String ouId = input.getOuId();
        String upperLineId = input.getUpperLineId();
        String lineName = input.getLineName();
        String gridTypeKey = input.getGridTypeKey();
        String lineTypeKey = input.getLineTypeKey();
        String strategyId = input.getStrategyId();
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        List<BasePowerLineEnt> selectList = mapper.selectList(new LambdaQueryWrapper<BasePowerLineEnt>()
                .select(BaseModel::getId, BasePowerLineEnt::getLineName)
                .eq(CharSequenceUtil.isNotBlank(lineId), BaseModel::getId, lineId)
                .eq(CharSequenceUtil.isNotBlank(ouId), BasePowerLineEnt::getOuId, ouId)
                .eq(CharSequenceUtil.isNotBlank(upperLineId), BasePowerLineEnt::getUpperLineId, upperLineId)
                .like(CharSequenceUtil.isNotBlank(lineName), BasePowerLineEnt::getLineName, lineName)
                .eq(CharSequenceUtil.isNotBlank(gridTypeKey), BasePowerLineEnt::getGridTypeKey, gridTypeKey)
                .eq(CharSequenceUtil.isNotBlank(lineTypeKey), BasePowerLineEnt::getLineTypeKey, lineTypeKey)
                .eq(CharSequenceUtil.isNotBlank(strategyId), BasePowerLineEnt::getStrategyId, strategyId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .orderByAsc(BasePowerLineEnt::getLineTypeKey, BasePowerLineEnt::getLineName)
        );
        List<DynamicQueryBasePowerLineOutput> lineOutputList = XCopyUtils.copyNewList(selectList, DynamicQueryBasePowerLineOutput.class);
        return XListResult.success(lineOutputList);
    }

    /**
     * 输电线路配置--井口列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    @XApiGet
    @XText("输电线路配置--井口列表")
    public XListResult<DynamicQueryBasePowerLineWellheadViewOutput> queryBasePowerLineWellheadList(XContext context, DynamicQueryBasePowerLineWellheadInput input) {
        BasePowerLineWellheadViewMapper wellheadViewMapper = context.getBean(BasePowerLineWellheadViewMapper.class);
        BasePowerLineWellheadView basePowerLineWellheadView = new BasePowerLineWellheadView();
        basePowerLineWellheadView.setLineId(input.getLineId());
        String wellheadId = input.getWellheadId();
        if (CharSequenceUtil.isNotBlank(wellheadId)) {
            basePowerLineWellheadView.setWellheadId(wellheadId);
        }
        List<BasePowerLineWellheadView> selectList = wellheadViewMapper.selectList(basePowerLineWellheadView);
        List<DynamicQueryBasePowerLineWellheadViewOutput> lineOutputList = XCopyUtils.copyNewList(selectList, DynamicQueryBasePowerLineWellheadViewOutput.class);
        if (CollUtil.isNotEmpty(lineOutputList)) {
            Map<String, String> dictMap = ServiceUtil.getDictMap(context, BusinessConstant.RUN_TYPE);
            for (DynamicQueryBasePowerLineWellheadViewOutput output : lineOutputList) {
                output.setRunTypeValue(
                        dictMap.get(output.getRunTypeKey())
                );
            }
        }
        return XListResult.success(lineOutputList);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查线路是否被间开制度引用
     *
     * @param context 上下文
     * @param lineId  线路id
     * @return boolean
     */
    private boolean checkLineIsReference(XContext context, String lineId) {
        ISpaceInstitutionDetailCloudService contextBean = context.getBean(ISpaceInstitutionDetailCloudService.class);
        XSingleResult<Long> result = contextBean.getInstitutionLineCountByLineId(context, lineId);
        result.throwIfFail();
        return result.getResult() > 0;
    }

    /**
     * 通过组织id获取系统组织名称
     *
     * @param context 上下文
     * @param ouId    组织id
     * @return {@link String}
     */
    private String getSysOrgNameById(XContext context, String ouId) {
        SysOrganizationCloudService orgCloudService = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput viewInput = new GetSysOrganizationViewInput();
        viewInput.setId(ouId);
        XSingleResult<GetSysOrganizationViewOutput> organization = orgCloudService.getSysOrganizationById(context, viewInput);
        organization.throwIfFail();
        return Optional.ofNullable(organization.getResult())
                .map(GetSysOrganizationViewOutput::getOuName)
                .orElse(null);
    }

    /**
     * 通过组织机构ID集合获取详情列表
     *
     * @param context 上下文
     * @param ouIds   ou id
     * @return {@link Map}<{@link String}, {@link String}>
     */
    private Map<String, String> getOuMapByOuIds(XContext context, List<String> ouIds) {
        GetOuListByOuIdsInput input = new GetOuListByOuIdsInput();
        input.setOuIdList(ouIds);
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        XListResult<GetOuListTreeOutput> ouListByOuIds = organizationCloudService.getOuListByOuIds(context, input);
        ouListByOuIds.throwIfFail();
        return ouListByOuIds.getResult().stream()
                .collect(Collectors.toMap(
                        GetOuListTreeOutput::getOuId, GetOuListTreeOutput::getOuName
                ));
    }

    /**
     * 通过线路ID获取线路信息
     *
     * @param lineId 行id
     * @param mapper 映射器
     * @return {@link BasePowerLineEnt}
     */
    private BasePowerLineEnt selectOneByLineId(String lineId, BasePowerLineMapper mapper) {
        return mapper.selectOne(new LambdaQueryWrapper<BasePowerLineEnt>()
                .eq(BaseModel::getId, lineId));
    }

    /**
     * 保存线路关联信息
     *
     * @param context        上下文
     * @param lineId         线路id
     * @param ouId           ou-id
     * @param session        一场
     * @param wellheadInputs 井口输入
     * @param plantInputs    电厂投入
     * @param storageInputs  存储输入
     * @param dieselInputs   柴油机输入
     */
    private void saveLineRelation(XContext context, String lineId, String ouId, PpsUserSession session,
                                  List wellheadInputs, List plantInputs, List storageInputs, List dieselInputs) {
        //新增井口配置
        if (CollUtil.isNotEmpty(wellheadInputs)) {
            BasePowerLineWellheadViewMapper wellheadMapper = context.getBean(BasePowerLineWellheadViewMapper.class);
            List<BasePowerLineWellheadView> wellheads = new ArrayList<>(32);
            for (BasePowerLineWellheadView wellhead : (List<BasePowerLineWellheadView>) XCopyUtils.copyNewList(wellheadInputs, BasePowerLineWellheadView.class)) {
                wellhead.setLineId(lineId);
                wellhead.setOuId(ouId);
                BaseUtils.setBaseModelDefault(wellhead, session);
                wellhead.setId(BaseUtils.getUUID());
                wellheads.add(wellhead);
            }
            wellheadMapper.batchInsertList(wellheads);
        }
        //新增光伏配置
        if (CollUtil.isNotEmpty(plantInputs)) {
            BasePowerLinePlantViewMapper plantMapper = context.getBean(BasePowerLinePlantViewMapper.class);
            List<BasePowerLinePlantView> plants = new ArrayList<>(32);
            for (BasePowerLinePlantView plant : (List<BasePowerLinePlantView>) XCopyUtils.copyNewList(plantInputs, BasePowerLinePlantView.class)) {
                plant.setLineId(lineId);
                plant.setOuId(ouId);
                BaseUtils.setBaseModelDefault(plant, session);
                plant.setId(UUID.randomUUID().toString());
                plants.add(plant);
            }
            plantMapper.batchInsertList(plants);
        }
        //新增储能配置
        if (CollUtil.isNotEmpty(storageInputs)) {
            BasePowerLineStorageViewMapper storageMapper = context.getBean(BasePowerLineStorageViewMapper.class);
            List<BasePowerLineStorageView> storages = new ArrayList<>(32);
            for (BasePowerLineStorageView storage : (List<BasePowerLineStorageView>) XCopyUtils.copyNewList(storageInputs, BasePowerLineStorageView.class)) {
                storage.setLineId(lineId);
                storage.setOuId(ouId);
                BaseUtils.setBaseModelDefault(storage, session);
                storage.setId(UUID.randomUUID().toString());
                storages.add(storage);
            }
            storageMapper.batchInsertList(storages);
        }
        //新增柴发配置
        if (CollUtil.isNotEmpty(dieselInputs)) {
            BasePowerLineDieselViewMapper dieselMapper = context.getBean(BasePowerLineDieselViewMapper.class);
            List<BasePowerLineDieselView> diesels = new ArrayList<>(32);
            for (BasePowerLineDieselView diesel : (List<BasePowerLineDieselView>) XCopyUtils.copyNewList(dieselInputs, BasePowerLineDieselView.class)) {
                diesel.setLineId(lineId);
                diesel.setOuId(ouId);
                BaseUtils.setBaseModelDefault(diesel, session);
                diesel.setId(UUID.randomUUID().toString());
                diesels.add(diesel);
            }
            dieselMapper.batchInsertList(diesels);
        }
    }
}
