package pps.core.base.service;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.core.base.entity.ThirdWeatherDataEnt;
import pps.core.base.mapper.ThirdWeatherDataMapper;
import pps.core.base.service.data.third_weather_data.GetThirdWeatherDataInput;
import pps.core.base.service.data.third_weather_data.GetThirdWeatherDataOutput;
import pps.core.common.constant.BusinessConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;

import java.io.File;
import java.math.BigDecimal;

/**
 * 第三方天气模块
 *
 * @author ZWT
 * @date 2024/05/21 11:19
 */
@XText("第三方天气模块")
@XService
public class ThirdWeatherDataService {

    /**
     * 今日天气
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetThirdWeatherDataOutput }>
     */
    @XText("能耗分析--今日天气")
    @XApiGet
    public XSingleResult<GetThirdWeatherDataOutput> todayWeather(XContext context, GetThirdWeatherDataInput input) {
        GetThirdWeatherDataOutput output;
        GetSysOrganizationViewOutput orgById = this.getOrgById(context, input.getOuId());
        if (ObjectUtil.isNotNull(orgById) && CharSequenceUtil.equals(orgById.getOuName(), "第二采油厂")) {
            ThirdWeatherDataMapper mapper = context.getBean(ThirdWeatherDataMapper.class);
            ThirdWeatherDataEnt ent = mapper.selectOne(new QueryWrapper<ThirdWeatherDataEnt>()
                    .select("wep", "tem", "wns_100m", "dni")
                    .lambda()
                    .ge(ThirdWeatherDataEnt::getForecastTime, DateUtil.date())
                    .orderByAsc(ThirdWeatherDataEnt::getForecastTime)
                    .last(BusinessConstant.LAST_LIMIT)
            );
            output = XCopyUtils.copyNewObject(ent, GetThirdWeatherDataOutput.class);
        } else {
            output = GetThirdWeatherDataOutput.builder()
                    .wep(9999)
                    .tem(BigDecimal.ZERO)
                    .wns100m(BigDecimal.ZERO)
                    .dni(BigDecimal.ZERO)
                    .build();
        }
        output.setIconPath(this.getIconPath(output.getWep()));
        return XSingleResult.success(output);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 按id获取组织
     *
     * @param context 上下文
     * @param ouId    ou-id
     * @return {@link GetSysOrganizationViewOutput }
     */
    private GetSysOrganizationViewOutput getOrgById(XContext context, String ouId) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        XSingleResult<GetSysOrganizationViewOutput> result = organizationCloudService.getSysOrganizationById(context, GetSysOrganizationViewInput.builder()
                .id(ouId)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取图标路径
     *
     * @param wep wep
     * @return {@link String }
     */
    private String getIconPath(Integer wep) {
        return XStartup.getCurrent().getProperty("weather.file.icon") + File.separator + wep + "@1x.png";
    }
}
