package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.BasePhotovoltaicPlantCloudService;
import pps.cloud.base.service.IBaseEnergyStorageCloudService;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.data.base_energy_storage_device.DynamicQueryBaseEnergyStorageDeviceInput;
import pps.cloud.base.service.data.base_energy_storage_device.DynamicQueryBaseEnergyStorageDeviceOutput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantViewOutput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.base.service.data.config_oil_field.GetConfigOilFieldOutput;
import pps.cloud.space.service.IDailyElectricityTrendCloudService;
import pps.cloud.space.service.data.line_daily_electricity_trend.GetLineDailyElectricityTrendInput;
import pps.cloud.space.service.data.line_daily_electricity_trend.GetLineDailyElectricityTrendOutput;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.data.sys_area.GetSysAreaInput;
import pps.cloud.system.service.data.sys_area.GetSysAreaOutput;
import pps.cloud.system.service.data.sys_organization.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import pps.core.prediction.entity.*;
import pps.core.prediction.enums.BusinessError;
import pps.core.prediction.mapper.*;
import pps.core.prediction.service.data.energy_consumption_analysis.DynamicQueryEnergyConsumptionAnalysisViewOutput;
import pps.core.prediction.service.data.home_page.*;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 首页
 *
 * @author ZWT
 * @date 2024/03/25 17:54
 */
@XText("首页模块")
@XService
public class HomePageService {

    /**
     * 首页模块--井场列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetStationViewOutput}>
     */
    @XText("首页模块--井场列表")
    @XApiGet
    public XListResult<GetStationViewOutput> getStationList(XContext context, GetStationViewInput input) {
        List<GetStationViewOutput> outputs;
        String stationId = input.getStationId();
        if (CharSequenceUtil.isBlank(stationId)) {
            stationId = context.getSession(PpsUserSession.class).getOuId();
            List<GetSysOrganizationViewOutput> ouList = ServiceUtil.getOrgIdsByPath(context, GetAllOuListByOuIdInput.builder()
                    .ouId(stationId)
                    .ouLevel(BusinessConstant.PLANT_FLAG)
                    .build());
            outputs = new ArrayList<>(ouList.size());
            ouList.forEach(item -> {
                outputs.add(GetStationViewOutput.builder()
                        .stationId(item.getId())
                        .stationName(item.getOuName())
                        .longitude(item.getLongitude())
                        .latitude(item.getLatitude())
                        .build());
            });
        } else {
            //查电站
            List<GetBasePhotovoltaicPlantCloudOutput> plantList = this.getPlantList(context, stationId, null);
            outputs = new ArrayList<>(plantList.size());
            plantList.forEach(item -> {
                outputs.add(GetStationViewOutput.builder()
                        .stationId(item.getId())
                        .stationName(item.getStationName())
                        .longitude(item.getLongitude())
                        .latitude(item.getLatitude())
                        .build());
            });
        }
        return XListResult.success(outputs);
    }

    /**
     * 首页模块--总览
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetOverviewViewOutput}>
     */
    @XText("首页模块--总览")
    @XApiGet
    public XSingleResult<GetOverviewViewOutput> getOverview(XContext context, GetStationViewInput input) {
        String stationId = input.getStationId();
        List<GetBasePhotovoltaicPlantCloudOutput> plantList;
        BigDecimal photovoltaicPower;
        BigDecimal storagePower;
        if (CharSequenceUtil.isBlank(stationId)) {
            stationId = context.getSession(PpsUserSession.class).getOuId();
        }
        List<String> ouIds = ServiceUtil.getOrgIdsByPath(context, stationId);
        plantList = this.getPlantList(context, ouIds);
        switch (ServiceUtil.getOilFieldCode(context)) {
            case BusinessConstant.ENV_CQ:
                switch (stationId) {
                    case "bb1bff9d-7068-48c9-9be8-c80eb0d49152":
                        //第一采油场
                        photovoltaicPower = new BigDecimal("226.12");
                        storagePower = new BigDecimal("852.96");
                        break;
                    case "2c4c9d1c-ad55-4c14-b889-4dc33e3461e2":
                        //第二采油场
                        photovoltaicPower = new BigDecimal("2650.86");
                        storagePower = BigDecimal.ZERO;
                        break;
                    case "0ffe6c33-ea02-476c-b28a-fd978184b2b3":
                        //第五采油场
                        photovoltaicPower = new BigDecimal("1178.28");
                        storagePower = BigDecimal.ZERO;
                        break;
                    default:
                        photovoltaicPower = new BigDecimal("3914.46");
                        storagePower = new BigDecimal("852.96");
                }
                break;
            case BusinessConstant.ENV_SY:
                List<DynamicQueryBaseEnergyStorageDeviceOutput> storageList = this.getStorageList(context, DynamicQueryBaseEnergyStorageDeviceInput.builder()
                        .ouIds(ouIds)
                        .build());
                photovoltaicPower = plantList.stream()
                        .map(GetBasePhotovoltaicPlantCloudOutput::getTotalPower)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                storagePower = storageList.stream()
                        .map(DynamicQueryBaseEnergyStorageDeviceOutput::getApparatusCapacity)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                break;
            default:
                plantList = Collections.emptyList();
                photovoltaicPower = BigDecimal.ZERO;
                storagePower = BigDecimal.ZERO;
        }
        List<DynamicQueryBaseWellheadOutput> wellList = this.getWellList(context, ouIds);
        List<DynamicQueryBasePowerLineOutput> lineList = ServiceUtil.getPowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .lineTypeKey("10")
                .ouIdList(ouIds)
                .build());
        return XSingleResult.success(GetOverviewViewOutput.builder()
                .wellNumber(wellList.size())
                .stationNumber(plantList.size())
                .lineNumber(lineList.size())
                .operatingLoad(wellList.stream()
                        .map(DynamicQueryBaseWellheadOutput::getRatedPower)
                        .reduce(BigDecimal.ZERO, BigDecimal::add))
                .photovoltaicPower(photovoltaicPower)
                .storagePower(storagePower)
                .build());
    }

    /**
     * 首页模块--井场实时监控
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetWellOverviewViewOutput}>
     */
    @XText("首页模块--井场实时监控")
    @XApiGet
    public XSingleResult<GetWellOverviewViewOutput> getWellOverview(XContext context, GetStationViewInput input) {
        DateTime beginOfDay = DateUtil.beginOfDay(DateUtil.offsetDay(DateUtil.date(), -3));
        String stationId = input.getStationId();
        BigDecimal cumulativeProduction = BigDecimal.ZERO;
        BigDecimal powerGeneration = BigDecimal.ZERO;
        BigDecimal powerConsumption = BigDecimal.ZERO;
        if (CharSequenceUtil.isBlank(stationId)) {
            stationId = context.getSession(PpsUserSession.class).getOuId();
        }
        List<String> ouIds = ServiceUtil.getOrgIdsByPath(context, stationId);
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        switch (oilFieldCode) {
            case BusinessConstant.ENV_CQ:
                //实时站
                Set<String> set = this.getPlantList(context, ouIds).stream()
                        .map(GetBasePhotovoltaicPlantCloudOutput::getStationName)
                        .collect(Collectors.toSet());
                ThirdDailyAccumulationUpdateMapper mapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
                ThirdDailyAccumulationUpdateEnt ent = mapper.selectOne(new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                        .select("IFNULL( ROUND( SUM( photovoltaic_power ), 2 ), 0 ) AS photovoltaic_power",
                                "IFNULL( ROUND( SUM( daily_electricity_consumption ), 2 ), 0 ) AS daily_electricity_consumption",
                                "IFNULL( ROUND( SUM( daily_liquid_production ), 2 ), 0 ) AS daily_liquid_production")
                        .lambda()
                        .eq(ThirdDailyAccumulationUpdateEnt::getSaveDate, beginOfDay)
                        .in(ThirdDailyAccumulationUpdateEnt::getStationName, set)
                );
                if (ObjectUtil.isNotNull(ent)) {
                    cumulativeProduction = ent.getDailyLiquidProduction();
                    powerGeneration = ent.getPhotovoltaicPower();
                    powerConsumption = ent.getDailyElectricityConsumption();
                }
                break;
            case BusinessConstant.ENV_SY:
                break;
            default:
        }
        //实时井口
        int openWellNumber = 0, stopWellNumber = 0, openJCNumber = 0, openLCNumber = 0, stopJCNumber = 0, stopLCNumber = 0;
        List<DynamicQueryBaseWellheadOutput> list = this.getWellList(context, ouIds);
        Map<String, String> collect = list.stream()
                .collect(Collectors.toMap(DynamicQueryBaseWellheadOutput::getWellNumber, DynamicQueryBaseWellheadOutput::getRunTypeKey));
        ThirdCurrentWellConditionMapper wellMapper = context.getBean(ThirdCurrentWellConditionMapper.class);
        List<ThirdCurrentWellConditionEnt> wellList = wellMapper.selectList(new LambdaQueryWrapper<ThirdCurrentWellConditionEnt>()
                .in(ThirdCurrentWellConditionEnt::getWellNumber, collect.keySet())
                .ge(ThirdCurrentWellConditionEnt::getUpdateTime, beginOfDay)
                .eq(ThirdCurrentWellConditionEnt::getSystemSource, oilFieldCode)
        );
        if (CollUtil.isNotEmpty(wellList)) {
            for (ThirdCurrentWellConditionEnt well : wellList) {
                if (collect.containsKey(well.getWellNumber())) {
                    if (CharSequenceUtil.equals(well.getWellStatus(), BusinessConstant.OPEN_WELL)) {
                        openWellNumber++;
                        if (CharSequenceUtil.equals(collect.get(well.getWellNumber()), BusinessConstant.INTERVAL_PUMPING_WELL)) {
                            openJCNumber++;
                        } else {
                            openLCNumber++;
                        }
                    } else {
                        stopWellNumber++;
                        if (CharSequenceUtil.equals(collect.get(well.getWellNumber()), BusinessConstant.INTERVAL_PUMPING_WELL)) {
                            stopJCNumber++;
                        } else {
                            stopLCNumber++;
                        }
                    }
                }
            }
        }
        return XSingleResult.success(GetWellOverviewViewOutput.builder()
                .openWellNumber(openWellNumber)
                .stopWellNumber(stopWellNumber)
                .openJCNumber(openJCNumber)
                .openLCNumber(openLCNumber)
                .stopJCNumber(stopJCNumber)
                .stopLCNumber(stopLCNumber)
                .cumulativeProduction(cumulativeProduction)
                .powerGeneration(powerGeneration)
                .powerConsumption(powerConsumption)
                .build());
    }

    /**
     * 首页模块--用能分析
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryEnergyConsumptionAnalysisViewOutput}>
     */
    @XText("首页模块--用能分析")
    @XApiGet
    public XListResult<GetPowerAnalyseOutput> getPowerAnalyse(XContext context, GetStationViewInput input) {
        String stationId = input.getStationId();
        if (CharSequenceUtil.isBlank(stationId)) {
            stationId = context.getSession(PpsUserSession.class).getOuId();
        }
        List<String> ouIds = ServiceUtil.getOrgIdsByPath(context, stationId);
        //查组织下所有井
        List<DynamicQueryBaseWellheadOutput> wellList = this.getWellList(context, ouIds);
        //查组织下所有电站
        List<GetBasePhotovoltaicPlantCloudOutput> plantList = this.getPlantList(context, ouIds);
        //查前七日用能分析
        DateTime endTime = DateUtil.beginOfDay(DateUtil.date());
        DateTime startTime = DateUtil.offsetDay(endTime, -7);
        ThirdWellAvgActivePowerViewMapper mapper = context.getBean(ThirdWellAvgActivePowerViewMapper.class);
        ThirdActivePowerViewMapper dailyUpdateMapper = context.getBean(ThirdActivePowerViewMapper.class);
        //当前配置
        List<ThirdWellAvgActivePowerView> avgList;
        List<ThirdActivePowerView> powerList;
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        //用电功率
        avgList = mapper.selectPowerConsumptionHour(ThirdWellAvgActivePowerView.builder()
                .wellNumbers(wellList.stream()
                        .map(DynamicQueryBaseWellheadOutput::getWellNumber)
                        .collect(Collectors.toList()))
                .startTime(startTime)
                .endTime(endTime)
                .systemSource(oilFieldCode)
                .build());
        //发电功率
        powerList = dailyUpdateMapper.selectGeneratedPowerHour(ThirdActivePowerView.builder()
                .stationNames(plantList.stream()
                        .map(GetBasePhotovoltaicPlantCloudOutput::getStationName)
                        .collect(Collectors.toList()))
                .startTime(startTime)
                .endTime(endTime)
                .systemSource(oilFieldCode)
                .build());
        Map<Date, BigDecimal> avgMap;
        if (CollUtil.isNotEmpty(avgList)) {
            avgMap = avgList.stream()
                    .collect(Collectors.toMap(ThirdWellAvgActivePowerView::getInputTime, ThirdWellAvgActivePowerView::getAvgActivePower));
        } else {
            avgMap = Collections.emptyMap();
        }
        Map<Date, BigDecimal> powerMap;
        if (CollUtil.isNotEmpty(powerList)) {
            powerMap = powerList.stream()
                    .collect(Collectors.toMap(ThirdActivePowerView::getCreateDate, ThirdActivePowerView::getPhotovoltaicPower));
        } else {
            powerMap = Collections.emptyMap();
        }
        //封装结果
        List<DateTime> rangeToList = DateUtil.rangeToList(startTime, endTime, DateField.HOUR_OF_DAY);
        List<GetPowerAnalyseOutput> outputs = new ArrayList<>(rangeToList.size());
        BigDecimal powerGeneration;
        BigDecimal powerConsumption;
        BigDecimal defaultPower = CharSequenceUtil.equals(oilFieldCode, BusinessConstant.ENV_CQ) ? BigDecimal.ZERO : null;
        for (DateTime dateTime : rangeToList) {
            powerConsumption = avgMap.getOrDefault(dateTime, defaultPower);
            powerGeneration = powerMap.getOrDefault(dateTime, BigDecimal.ZERO);
            outputs.add(GetPowerAnalyseOutput.builder()
                    .dateFormat(dateTime.toString(BusinessConstant.DATE_FORMAT_ALL))
                    .powerGeneration(powerGeneration)
                    .powerConsumption(powerConsumption)
                    .build());
        }
        return XListResult.success(outputs);
    }

    /**
     * 井场用能分析(双坐标轴)
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetPowerAnalyseTrendOutput }>
     */
    @XText("首页模块--井场用能分析")
    @XApiGet
    public XSingleResult<GetPowerAnalyseTrendOutput> getPowerAnalyseTrend(XContext context, GetStationViewInput input) {
        String stationId = input.getStationId();
        if (CharSequenceUtil.isBlank(stationId)) {
            stationId = context.getSession(PpsUserSession.class).getOuId();
        }
        List<String> ouIds = ServiceUtil.getOrgIdsByPath(context, stationId);
        //查组织下所有井
        List<DynamicQueryBaseWellheadOutput> wellList = this.getWellList(context, ouIds);
        //查组织下所有电站
        List<GetBasePhotovoltaicPlantCloudOutput> plantList = this.getPlantList(context, ouIds);
        //查前七日用能分析
        DateTime endTime = DateUtil.beginOfDay(DateUtil.date());
        DateTime startTime = DateUtil.offsetDay(endTime, -7);
        ThirdWellAvgActivePowerViewMapper mapper = context.getBean(ThirdWellAvgActivePowerViewMapper.class);
        ThirdActivePowerViewMapper dailyUpdateMapper = context.getBean(ThirdActivePowerViewMapper.class);
        //当前配置
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        //用电功率
        List<ThirdWellAvgActivePowerView> avgList = mapper.selectPowerConsumptionHour(ThirdWellAvgActivePowerView.builder()
                .wellNumbers(wellList.stream()
                        .map(DynamicQueryBaseWellheadOutput::getWellNumber)
                        .collect(Collectors.toList()))
                .startTime(startTime)
                .endTime(endTime)
                .systemSource(oilFieldCode)
                .build());
        //发电功率
        List<ThirdActivePowerView> powerList = dailyUpdateMapper.selectGeneratedPowerHour(ThirdActivePowerView.builder()
                .stationNames(plantList.stream()
                        .map(GetBasePhotovoltaicPlantCloudOutput::getStationName)
                        .collect(Collectors.toList()))
                .startTime(startTime)
                .endTime(endTime)
                .systemSource(oilFieldCode)
                .build());
        //封装
        GetPowerAnalyseTrendOutput output = new GetPowerAnalyseTrendOutput();
        if (CollUtil.isNotEmpty(avgList)) {
            List<GetPowerConsumptionTrendOutput> consumptionList = new ArrayList<>(avgList.size());
            for (ThirdWellAvgActivePowerView view : avgList) {
                consumptionList.add(
                        new GetPowerConsumptionTrendOutput(DateUtil.format(view.getInputTime(), BusinessConstant.DATE_FORMAT_HOUR), view.getAvgActivePower())
                );
            }
            output.setConsumptionList(consumptionList);
        } else {
            output.setConsumptionList(Collections.emptyList());
        }
        if (CollUtil.isNotEmpty(powerList)) {
            List<GetPowerGenerationTrendOutput> generationList = new ArrayList<>(powerList.size());
            for (ThirdActivePowerView view : powerList) {
                generationList.add(
                        new GetPowerGenerationTrendOutput(DateUtil.format(view.getCreateDate(), BusinessConstant.DATE_FORMAT_HOUR), view.getPhotovoltaicPower())
                );
            }
            output.setGenerationList(generationList);
        } else {
            output.setGenerationList(Collections.emptyList());
        }
        return XSingleResult.success(output);
    }

    /**
     * 累积用电
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSumPowerConsumptionOutput}>
     */
    @XText("首页模块--累积用电")
    @XApiGet
    public XSingleResult<GetSumPowerConsumptionOutput> getSumPowerConsumption(XContext context, GetStationViewInput input) {
        String stationId = input.getStationId();
        DateTime startTime = DateUtil.beginOfDay(DateUtil.yesterday());
        List<GetStationPowerConsumption> stationList;
        BigDecimal powerConsumption = BigDecimal.ZERO;
        EnergyConsumptionAnalysisMapper mapper = context.getBean(EnergyConsumptionAnalysisMapper.class);
        if (CharSequenceUtil.isBlank(stationId)) {
            stationId = context.getSession(PpsUserSession.class).getOuId();
            Map<String, String> collect = ServiceUtil.getOrgIdsByPath(context, GetAllOuListByOuIdInput.builder()
                            .ouId(stationId)
                            .ouLevel(BusinessConstant.PLANT_FLAG)
                            .build()).stream()
                    .collect(Collectors.toMap(GetSysOrganizationViewOutput::getId, GetSysOrganizationViewOutput::getOuName));
            Map<String, List<DynamicQueryBasePowerLineOutput>> listMap = this.getBasePowerLineListByOuIdList(context, collect.keySet()).stream()
                    .collect(Collectors.groupingBy(DynamicQueryBasePowerLineOutput::getOuId));
            stationList = new ArrayList<>(4);
            for (Map.Entry<String, List<DynamicQueryBasePowerLineOutput>> entry : listMap.entrySet()) {
                List<String> lineNameList = entry.getValue().stream()
                        .map(DynamicQueryBasePowerLineOutput::getLineName)
                        .collect(Collectors.toList());
                EnergyConsumptionAnalysisEnt ent = mapper.selectOne(new QueryWrapper<EnergyConsumptionAnalysisEnt>()
                        .select("IFNULL( SUM( power_consumption ), 0 ) AS power_consumption")
                        .lambda()
                        .in(EnergyConsumptionAnalysisEnt::getStationName, lineNameList)
                        .eq(EnergyConsumptionAnalysisEnt::getDataDate, startTime)
                );
                stationList.add(GetStationPowerConsumption.builder()
                        .stationName(collect.get(entry.getKey()))
                        .powerConsumption(ent.getPowerConsumption())
                        .build());
                powerConsumption = powerConsumption.add(ent.getPowerConsumption());
            }
        } else {
            List<String> lineNameList = this.getBasePowerLineListByOuIdList(context, CollUtil.list(false, stationId)).stream()
                    .map(DynamicQueryBasePowerLineOutput::getLineName)
                    .collect(Collectors.toList());
            if (CollUtil.isNotEmpty(lineNameList)) {
                List<EnergyConsumptionAnalysisEnt> list = mapper.selectList(new QueryWrapper<EnergyConsumptionAnalysisEnt>()
                        .select("IFNULL( SUM( power_consumption ), 0 ) AS power_consumption", "station_name")
                        .lambda()
                        .in(EnergyConsumptionAnalysisEnt::getStationName, lineNameList)
                        .eq(EnergyConsumptionAnalysisEnt::getDataDate, startTime)
                        .groupBy(EnergyConsumptionAnalysisEnt::getStationName)
                );
                stationList = new ArrayList<>(list.size());
                for (EnergyConsumptionAnalysisEnt ent : list) {
                    stationList.add(GetStationPowerConsumption.builder()
                            .stationName(ent.getStationName())
                            .powerConsumption(ent.getPowerConsumption())
                            .build());
                    powerConsumption = powerConsumption.add(ent.getPowerConsumption());
                }
            } else {
                stationList = Collections.emptyList();
            }
        }
        return XSingleResult.success(GetSumPowerConsumptionOutput.builder()
                .powerConsumption(powerConsumption)
                .stationList(stationList)
                .build());
    }

    /**
     * 光伏实时监控
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetPowerPredictionOutput}>
     */
    @XText("首页模块--光伏实时监控")
    @XApiGet
    public XListResult<GetPowerPredictionOutput> getPowerPrediction(XContext context, GetStationViewInput input) {
        String plantId = input.getPlantId();
        List<GetBasePhotovoltaicPlantCloudOutput> plantList;
        List<String> plantIds;
        if (CharSequenceUtil.isNotBlank(plantId)) {
            input.setStationId(null);
            plantList = this.getPlantList(context, null, plantId);
        } else {
            List<String> ouIds = ServiceUtil.getOrgIdsByPath(context, context.getSession(PpsUserSession.class).getOuId());
            plantList = this.getPlantList(context, ouIds);
        }
        plantIds = plantList.stream()
                .map(GetBasePhotovoltaicPlantCloudOutput::getId)
                .collect(Collectors.toList());
        DateTime date = DateUtil.date();
        DateTime day = DateUtil.beginOfDay(date);
        //判断是否需要启用演示配置
        GetConfigOilFieldOutput currentConfig = ServiceUtil.getCurrentConfig(context);
        Date showTime = null;
        if (ObjectUtil.isNotNull(currentConfig.getDemoDate())) {
            day = currentConfig.getIsDemo().equals(0) ? DateUtil.date(currentConfig.getDemoDate()) : day;
            showTime = DateUtil.parse(day.toString(BusinessConstant.DATE_FORMAT_DAY) + " " + date.toString(BusinessConstant.TIME_FORMAT));
        }
        //预测发电量
        PredictedPowerViewMapper mapper = context.getBean(PredictedPowerViewMapper.class);
        Map<String, BigDecimal> shortPowerMap = mapper.selectShortPowerHourList(PredictedPowerView.builder()
                        .startTime(day.toString(BusinessConstant.DATE_FORMAT_DAY))
                        .endTime(DateUtil.offsetDay(day, 1).toString(BusinessConstant.DATE_FORMAT_DAY))
                        .plantIds(plantIds)
                        .plantId(plantId)
                        .build()).stream()
                .collect(Collectors.toMap(PredictedPowerView::getHourTime, PredictedPowerView::getPower));
        //实际功率
        ThirdActivePowerDailyUpdateViewMapper nowMapper = context.getBean(ThirdActivePowerDailyUpdateViewMapper.class);
        Map<Integer, BigDecimal> powerMap = nowMapper.selectEnergyUseCurve(ThirdActivePowerDailyUpdateView.builder()
                        .stationIds(plantIds)
                        .stationId(plantId)
                        .saveDate(day)
                        .createDate(showTime)
                        .systemSource(ServiceUtil.getOilFieldCode(context))
                        .build()).stream()
                .collect(Collectors.toMap(ThirdActivePowerDailyUpdateView::getHourNumber, ThirdActivePowerDailyUpdateView::getPhotovoltaicPower));
        //封装结果
        List<GetPowerPredictionOutput> outputs = new ArrayList<>(24);
        BigDecimal predictPower;
        BigDecimal actualPower;
        for (int i = 0; i < 24; i++) {
            String value = String.valueOf(i);
            predictPower = shortPowerMap.getOrDefault(value, BigDecimal.ZERO);
            actualPower = powerMap.getOrDefault(i, BigDecimal.ZERO);
            outputs.add(
                    GetPowerPredictionOutput.builder()
                            .hourNumber(i)
                            .predictPower(predictPower)
                            .actualPower(actualPower)
                            .build()
            );
        }
        return XListResult.success(outputs);
    }

    /**
     * 井场效果评价
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetIncomeAnalysisOutput}>
     */
    @XText("首页模块--井场效果评价")
    @XApiGet
    public XSingleResult<GetIncomeAnalysisOutput> getIncomeAnalysis(XContext context, GetStationViewInput input) {
        String stationId = input.getStationId();
        boolean isOffGrid = false;
        Set<String> lineNameList;
        BigDecimal powerGeneration = BigDecimal.ZERO;
        BigDecimal dailyElectricityConsumption = BigDecimal.ZERO;
        BigDecimal energyStorageCapacity = BigDecimal.ZERO;
        BigDecimal carbonReduction = BigDecimal.ZERO;
        BigDecimal economicBenefit = BigDecimal.ZERO;
        BigDecimal valleyElectricRate = BigDecimal.ZERO;
        BigDecimal accumulatedCharge = BigDecimal.ZERO;
        String greenElectricityRate = "-";
        String zeroGreenElectricityRate = "100";
        BigDecimal avoidPeakRate = BigDecimal.ZERO;
        if (CharSequenceUtil.isBlank(stationId)) {
            stationId = context.getSession(PpsUserSession.class).getOuId();
        }
        List<String> ouIds = ServiceUtil.getOrgIdsByPath(context, stationId);
        List<DynamicQueryBasePowerLineOutput> lineList = this.getLineList(context, DynamicQueryBasePowerLineInput.builder()
                .ouIdList(ouIds)
                .build());
        if (CollUtil.isNotEmpty(lineList)) {
            lineNameList = new HashSet<>(lineList.size());
            for (DynamicQueryBasePowerLineOutput line : lineList) {
                lineNameList.add(line.getLineName());
                if (CharSequenceUtil.equals(line.getGridTypeKey(), "0")) {
                    isOffGrid = true;
                }
            }
            String oilFieldCode = ServiceUtil.getOilFieldCode(context);
            ThirdDailyAccumulationUpdateMapper analysisMapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
            ThirdDailyAccumulationUpdateEnt analysisEnt = analysisMapper.selectOne(new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                    .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power",
                            "IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                            "IFNULL( SUM( daily_liquid_production ), 0 ) AS daily_liquid_production",
                            "IFNULL( SUM( energy_storage_discharge ), 0 ) AS energy_storage_discharge",
                            "IFNULL( SUM( in_place_consumption ), 0 ) AS in_place_consumption")
                    .lambda()
                    .in(ThirdDailyAccumulationUpdateEnt::getStationName, lineNameList)
                    .eq(ThirdDailyAccumulationUpdateEnt::getSystemSource, oilFieldCode)
            );
            //发电量
            powerGeneration = analysisEnt.getPhotovoltaicPower()
                    .divide(BusinessConstant.BIG_DECIMAL_10000, 2, RoundingMode.HALF_UP);
            //用电量
            dailyElectricityConsumption = analysisEnt.getDailyElectricityConsumption()
                    .divide(BusinessConstant.BIG_DECIMAL_10000, 2, RoundingMode.HALF_UP);
            //累计减碳量
            carbonReduction = analysisEnt.getPhotovoltaicPower()
                    .multiply(BusinessConstant.BIG_DECIMAL_6_67)
                    .divide(BusinessConstant.BIG_DECIMAL_10000, 2, RoundingMode.HALF_UP);
            //经济效益(万元):光伏累计发电*0.65-光伏累计发电*0.28
            economicBenefit = analysisEnt.getPhotovoltaicPower()
                    .multiply(BigDecimal.valueOf(0.65))
                    .subtract(analysisEnt.getPhotovoltaicPower()
                            .multiply(BigDecimal.valueOf(0.28)))
                    .divide(BusinessConstant.BIG_DECIMAL_10000, 2, RoundingMode.HALF_UP);
            //区分环境
            switch (oilFieldCode) {
                case BusinessConstant.ENV_CQ:
                    if (CharSequenceUtil.equals("bb1bff9d-7068-48c9-9be8-c80eb0d49152", stationId)) {
                        greenElectricityRate = "94.03";
                    } else {
                        //近零碳井场绿电占比:(累计就地消纳量/累计井场用电量*100%)+5
                        greenElectricityRate = BaseUtils.getRate(analysisEnt.getInPlaceConsumption(), analysisEnt.getDailyElectricityConsumption())
                                .add(BigDecimal.valueOf(5))
                                .toString();
                    }
                    //余电上网:发电量-就地消纳量
                    avoidPeakRate = analysisEnt.getPhotovoltaicPower()
                            .subtract(analysisEnt.getInPlaceConsumption())
                            .divide(BusinessConstant.BIG_DECIMAL_10000, 2, RoundingMode.HALF_UP);
                    //储能日累计放电量
                    energyStorageCapacity = analysisEnt.getEnergyStorageDischarge()
                            .divide(BusinessConstant.BIG_DECIMAL_10000, 2, RoundingMode.HALF_UP);
                    break;
                case BusinessConstant.ENV_SY:
                    //近零碳井场绿电占比:(累计就地消纳量/累计井场用电量*100%)+5
                    greenElectricityRate = BaseUtils.getRate(analysisEnt.getInPlaceConsumption(), analysisEnt.getDailyElectricityConsumption())
                            .add(BigDecimal.valueOf(5))
                            .toString();
                    //谷电占比:所有井的谷电时间开井时长/所有井开井时长保留2位小数
                    IDailyElectricityTrendCloudService bean = context.getBean(IDailyElectricityTrendCloudService.class);
                    XSingleResult<GetLineDailyElectricityTrendOutput> result = bean.querySumDailyElectricity(context, GetLineDailyElectricityTrendInput.builder()
                            .systemSource(oilFieldCode)
                            .stationNameList(lineNameList)
                            .build());
                    result.throwIfFail();
                    GetLineDailyElectricityTrendOutput lineDailyElectricity = result.getResult();
                    valleyElectricRate = BaseUtils.getRate(lineDailyElectricity.getValleyElectricOpenHour(), lineDailyElectricity.getSumRunDuration());
                    //累计电费
                    break;
                default:
            }
            //零碳井场绿电占比:有离网线路的默认100%
            zeroGreenElectricityRate = isOffGrid || ObjectUtil.isEmpty(stationId) ? zeroGreenElectricityRate : "-";
        }
        return XSingleResult.success(GetIncomeAnalysisOutput.builder()
                .absorptionRate(BusinessConstant.BIG_DECIMAL_100)
                .avoidPeakRate(avoidPeakRate)
                .energyStorageCapacity(energyStorageCapacity)
                .powerGeneration(powerGeneration)
                .greenElectricityRate(greenElectricityRate)
                .carbonReduction(carbonReduction)
                .economicBenefit(economicBenefit)
                .dailyElectricityConsumption(dailyElectricityConsumption)
                .zeroGreenElectricityRate(zeroGreenElectricityRate)
                .valleyElectricRate(valleyElectricRate)
                .accumulatedCharge(accumulatedCharge)
                .build());
    }

    /**
     * 采油厂详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetOilExtractionPlantOutput}>
     */
    @XText("首页模块--采油厂详情")
    @XApiGet
    public XSingleResult<GetOilExtractionPlantOutput> getOilExtractionPlantInfo(XContext context, GetStationViewInput input) {
        String stationId = input.getStationId();
        if (CharSequenceUtil.isBlank(stationId)) {
            return XSingleResult.error(context, BusinessError.MissingoilExtractionPlantID);
        }
        Integer intervalWellNumber = 0;
        Integer continuousWellNumber = 0;
        BigDecimal operatingLoad = BigDecimal.ZERO;
        //井口统计
        List<DynamicQueryBaseWellheadOutput> wellList = this.getWellList(context, stationId);
        for (DynamicQueryBaseWellheadOutput well : wellList) {
            if (CharSequenceUtil.equals(BusinessConstant.CONTINUOUS_PUMPING_WELL, well.getRunTypeKey())) {
                continuousWellNumber++;
            } else {
                intervalWellNumber++;
            }
            operatingLoad = operatingLoad.add(Optional.ofNullable(well.getRatedPower()).orElse(BigDecimal.ZERO));
        }
        //线路统计
        List<DynamicQueryBasePowerLineOutput> lineList = this.getLineList(context, stationId, "10", null);
        //光伏电站
        List<GetBasePhotovoltaicPlantCloudOutput> plantList = this.getPlantList(context, stationId, null);
        //储能电站
        List<DynamicQueryBaseEnergyStorageDeviceOutput> storageList = this.getStorageList(context, stationId);
        return XSingleResult.success(GetOilExtractionPlantOutput.builder()
                .wellNumber(wellList.size())
                .intervalWellNumber(intervalWellNumber)
                .continuousWellNumber(continuousWellNumber)
                .operatingLoad(operatingLoad.setScale(2, RoundingMode.HALF_UP))
                .lineNumber(lineList.size())
                .photovoltaicPower(plantList.stream()
                        .map(GetBasePhotovoltaicPlantCloudOutput::getTotalPower)
                        .reduce(BigDecimal.ZERO, BigDecimal::add)
                        .setScale(2, RoundingMode.HALF_UP))
                .storagePower(storageList.stream()
                        .map(DynamicQueryBaseEnergyStorageDeviceOutput::getApparatusCapacity)
                        .reduce(BigDecimal.ZERO, BigDecimal::add)
                        .setScale(2, RoundingMode.HALF_UP))
                .build());
    }

    /**
     * 线路图
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetBranchingProgramOutput}>
     */
    @XText("首页模块--线路图")
    @XApiGet
    public XListResult<GetBranchingProgramOutput> getBranchingProgram(XContext context, GetStationViewInput input) {
        String stationId = input.getStationId();
        if (CharSequenceUtil.isBlank(stationId)) {
            return XListResult.error(context, BusinessError.MissingoilExtractionPlantID);
        }
        //十千伏线路
        List<DynamicQueryBasePowerLineOutput> lineList = this.getLineList(context, stationId, "10", null);
        List<GetBranchingProgramOutput> outputs = new ArrayList<>(lineList.size());
        for (DynamicQueryBasePowerLineOutput line : lineList) {
            outputs.add(GetBranchingProgramOutput.builder()
                    .mainLineId(line.getId())
                    .mainLineName(line.getLineName())
                    .subLineList(this.getSubLineList(context, line.getId()))
                    .build());
        }
        return XListResult.success(outputs);
    }

    /**
     * 线路详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetLineViewOutput}>
     */
    @XText("首页模块--线路详情")
    @XApiGet
    public XSingleResult<GetLineViewOutput> getLineView(XContext context, GetLineViewInput input) {
        //查线路关联的井口
        BigDecimal operatingLoad = BigDecimal.ZERO;
        List<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList = this.getPowerLineWellheadList(context, input.getLineId());
        if (CollUtil.isNotEmpty(wellheadList)) {
            for (DynamicQueryBasePowerLineWellheadViewOutput well : wellheadList) {
                operatingLoad = operatingLoad.add(Optional.ofNullable(well.getRatedPower()).orElse(BigDecimal.ZERO));
            }
        }
        //查线路关联的电站
        List<DynamicQueryBasePowerLinePlantViewOutput> plantViewList = this.getPowerLinePlantViewList(context, input.getLineId());
        String areaName;
        if (CollUtil.isNotEmpty(plantViewList)) {
            areaName = this.getSysAreaPath(context, plantViewList.get(0).getAreaCode());
        } else {
            return XSingleResult.success(GetLineViewOutput.builder()
                    .wellList(wellheadList)
                    .operatingLoad(operatingLoad)
                    .cumulativeProduction(BigDecimal.ZERO)
                    .photovoltaicPower(BigDecimal.ZERO)
                    .powerGeneration(BigDecimal.ZERO)
                    .dailyElectricityConsumption(BigDecimal.ZERO)
                    .build());
        }
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        DateTime today = DateUtil.beginOfDay(DateUtil.date());
        ThirdDailyAccumulationUpdateMapper mapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
        ThirdDailyAccumulationUpdateEnt dailyAccumulation = mapper.selectOne(new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                .select("IFNULL( AVG( photovoltaic_power ), 0 ) AS photovoltaic_power",
                        "IFNULL( AVG( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                        "IFNULL( AVG( daily_liquid_production ), 0 ) AS daily_liquid_production",
                        "IFNULL( AVG( in_place_consumption ), 0 ) AS in_place_consumption")
                .lambda()
                .in(ThirdDailyAccumulationUpdateEnt::getStationName, plantViewList.stream()
                        .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                        .collect(Collectors.toList()))
                .eq(ThirdDailyAccumulationUpdateEnt::getSystemSource, oilFieldCode)
                .apply("DATE( create_date ) = {0}", today)
                .groupBy(ThirdDailyAccumulationUpdateEnt::getStationName)
        );
        BigDecimal cumulativeProduction;
        BigDecimal powerGeneration;
        BigDecimal dailyElectricityConsumption;
        if (ObjectUtil.isNull(dailyAccumulation)) {
            cumulativeProduction = BigDecimal.ZERO;
            powerGeneration = BigDecimal.ZERO;
            dailyElectricityConsumption = BigDecimal.ZERO;
        } else {
            cumulativeProduction = dailyAccumulation.getDailyLiquidProduction().setScale(2, RoundingMode.HALF_UP);
            powerGeneration = dailyAccumulation.getPhotovoltaicPower().setScale(2, RoundingMode.HALF_UP);
            dailyElectricityConsumption = dailyAccumulation.getDailyElectricityConsumption().setScale(2, RoundingMode.HALF_UP);
        }
        return XSingleResult.success(GetLineViewOutput.builder()
                .wellList(wellheadList)
                .operatingLoad(operatingLoad)
                .cumulativeProduction(cumulativeProduction)
                .photovoltaicPower(plantViewList.stream()
                        .map(DynamicQueryBasePowerLinePlantViewOutput::getTotalPower)
                        .reduce(BigDecimal.ZERO, BigDecimal::add)
                        .setScale(2, RoundingMode.HALF_UP))
                .powerGeneration(powerGeneration)
                .dailyElectricityConsumption(dailyElectricityConsumption)
                .areaName(areaName)
                .build());
    }

    /*------------------------------------------------- private ---------------------------------------------------------*/

    /**
     * 通过电站ID查井口
     *
     * @param context  上下文
     * @param plantIds 植物id
     * @return {@link List}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    private List<DynamicQueryBasePowerLineWellheadViewOutput> getPowerLineWellheadListByPlant(XContext context, List<String> plantIds) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineWellheadViewOutput> result = service.getPowerLineWellheadListByPlant(context, DynamicQueryBasePowerLineWellheadInput.builder()
                .plantIds(plantIds)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取区域路径
     *
     * @param context 上下文
     * @param areaId  区域id
     * @return {@link String}
     */
    private String getSysAreaPath(XContext context, Integer areaId) {
        SystemAreaService service = context.getBean(SystemAreaService.class);
        XSingleResult<GetSysAreaOutput> result = service.getSysAreaPath(context, GetSysAreaInput.builder()
                .id(areaId)
                .build());
        result.throwIfFail();
        return result.getResult().getName();
    }

    /**
     * 获取输电线电站列表
     *
     * @param context 上下文
     * @param lineId  线路id
     * @return {@link List}<{@link DynamicQueryBasePowerLinePlantViewOutput}>
     */
    private List<DynamicQueryBasePowerLinePlantViewOutput> getPowerLinePlantViewList(XContext context, String lineId) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLinePlantViewOutput> result = service.getPowerLinePlantViewList(context, DynamicQueryBasePowerLinePlantInput.builder()
                .lineId(lineId)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取输电线井口列表
     *
     * @param context 上下文
     * @param lineId  线路id
     * @return {@link List}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    private List<DynamicQueryBasePowerLineWellheadViewOutput> getPowerLineWellheadList(XContext context, String lineId) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineWellheadViewOutput> result = service.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                .lineId(lineId)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取输电线井口列表
     *
     * @param context 上下文
     * @param lineIds 线路ID
     * @return {@link List }<{@link DynamicQueryBasePowerLineWellheadViewOutput }>
     */
    private List<DynamicQueryBasePowerLineWellheadViewOutput> getPowerLineWellheadList(XContext context, List<String> lineIds) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineWellheadViewOutput> result = service.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                .lineIds(lineIds)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 查询组织列表下所有线路
     *
     * @param context  上下文
     * @param ouIdList ou-id列表
     * @return {@link List}<{@link DynamicQueryBasePowerLineOutput}>
     */
    private List<DynamicQueryBasePowerLineOutput> getBasePowerLineListByOuIdList(XContext context, Collection<String> ouIdList) {
        IBasePowerLineCloudService basePowerLineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> lineList = basePowerLineCloudService.getBasePowerLineListByOuIdList(context, DynamicQueryBasePowerLineInput.builder()
                .ouIdList(ouIdList)
                .build());
        lineList.throwIfFail();
        return lineList.getResult();
    }

    /**
     * 获取井口列表
     *
     * @param context   上下文
     * @param stationId 工作站id
     * @return {@link List}<{@link DynamicQueryBaseWellheadOutput}>
     */
    private List<DynamicQueryBaseWellheadOutput> getWellList(XContext context, String stationId) {
        IBaseWellheadCloudService service = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = service.queryBaseWellheadListByParam(context, DynamicQueryBaseWellheadInput.builder()
                .ouId(stationId)
                .build());
        result.throwIfFail();
        return result.getResult();
    }


    /**
     * 光伏电站列表
     *
     * @param context   上下文
     * @param stationId 工作站id
     * @param plantId   电站id
     * @return {@link List}<{@link GetBasePhotovoltaicPlantCloudOutput}>
     */
    private List<GetBasePhotovoltaicPlantCloudOutput> getPlantList(XContext context, String stationId, String plantId) {
        BasePhotovoltaicPlantCloudService service = context.getBean(BasePhotovoltaicPlantCloudService.class);
        XListResult<GetBasePhotovoltaicPlantCloudOutput> result = service.getBasePhotovoltaicPlantList(context, GetBasePhotovoltaicPlantCloudInput.builder()
                .ouId(stationId)
                .id(plantId)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取井口列表
     *
     * @param context 上下文
     * @param ouIds   ou-ids
     * @return {@link List }<{@link DynamicQueryBaseWellheadOutput }>
     */
    private List<DynamicQueryBaseWellheadOutput> getWellList(XContext context, List<String> ouIds) {
        IBaseWellheadCloudService service = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = service.queryBaseWellheadListByParam(context, DynamicQueryBaseWellheadInput.builder()
                .ouIds(ouIds)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 光伏电站列表
     *
     * @param context 上下文
     * @param ouIds   ou-ids
     * @return {@link List }<{@link GetBasePhotovoltaicPlantCloudOutput }>
     */
    private List<GetBasePhotovoltaicPlantCloudOutput> getPlantList(XContext context, List<String> ouIds) {
        BasePhotovoltaicPlantCloudService service = context.getBean(BasePhotovoltaicPlantCloudService.class);
        XListResult<GetBasePhotovoltaicPlantCloudOutput> result = service.getBasePhotovoltaicPlantList(context, GetBasePhotovoltaicPlantCloudInput.builder()
                .ouIds(ouIds)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取储能列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List }<{@link DynamicQueryBaseEnergyStorageDeviceOutput }>
     */
    private List<DynamicQueryBaseEnergyStorageDeviceOutput> getStorageList(XContext context, DynamicQueryBaseEnergyStorageDeviceInput input) {
        IBaseEnergyStorageCloudService service = context.getBean(IBaseEnergyStorageCloudService.class);
        XListResult<DynamicQueryBaseEnergyStorageDeviceOutput> result = service.queryBaseStorageListByParam(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 输电线路列表
     *
     * @param context     上下文
     * @param stationId   工作站id
     * @param lineTypeKey 线型键
     * @return {@link List}<{@link DynamicQueryBasePowerLineOutput}>
     */
    private List<DynamicQueryBasePowerLineOutput> getLineList(XContext context, String stationId, String lineTypeKey, String lineId) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> result = service.getBasePowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .lineId(lineId)
                .ouId(stationId)
                .lineTypeKey(lineTypeKey)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 输电线路列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List }<{@link DynamicQueryBasePowerLineOutput }>
     */
    private List<DynamicQueryBasePowerLineOutput> getLineList(XContext context, DynamicQueryBasePowerLineInput input) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> result = service.getBasePowerLineList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取子线路列表
     *
     * @param context     上下文
     * @param upperLineId 上级线路ID
     * @return {@link List}<{@link DynamicQueryBasePowerLineOutput}>
     */
    private List<DynamicQueryBasePowerLineOutput> getSubLineList(XContext context, String upperLineId) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> result = service.getBasePowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .upperLineId(upperLineId)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取储能列表
     *
     * @param context   上下文
     * @param stationId 工作站id
     * @return {@link List}<{@link DynamicQueryBaseEnergyStorageDeviceOutput}>
     */
    private List<DynamicQueryBaseEnergyStorageDeviceOutput> getStorageList(XContext context, String stationId) {
        IBaseEnergyStorageCloudService service = context.getBean(IBaseEnergyStorageCloudService.class);
        XListResult<DynamicQueryBaseEnergyStorageDeviceOutput> result = service.queryBaseStorageListByParam(context, DynamicQueryBaseEnergyStorageDeviceInput.builder()
                .ouId(stationId)
                .build());
        result.throwIfFail();
        return result.getResult();
    }
}